"""
Written by Matteo Dunnhofer - 2019

Class that defines the Actor Critic tracker model
"""
import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.autograd import Variable
from model.ResNets import ResNet18, ResNet18IF, ResNet50



class ActorCriticModel(torch.nn.Module):

    def __init__(self, cfg):
        super(ActorCriticModel, self).__init__()

        self.model_name = 'ActorCriticModel'

        self.cfg = cfg

        self.use_if = False # use CNN intermediate features
        self.lstm_layers = 1
        self.lstm_size = 512

        # cnn layers
        if not self.use_if:
            self.cnn_features = ResNet18(self.cfg)
            #self.cnn_features = VGGM_(self.cfg)

            self.fc = nn.Linear(8192 * 2, 512)
            #self.dp = nn.Dropout(0.2)
        else:
            self.cnn_features = ResNet18IF(self.cfg)

            self.fc = nn.Linear(81920, 512)


        self.fc2 = nn.Linear(512, 512)
        #self.dp2 = nn.Dropout(0.2)

        self.lstm = nn.LSTM(512, hidden_size=self.lstm_size, num_layers=self.lstm_layers)

        # actor
        self.actor_policy = nn.Linear(self.lstm_size, 4)

        # critic
        self.critic = nn.Linear(self.lstm_size, 1)


    def forward(self, x1, x2, state, device):
        """ 
        Function that executes the model 
        """
        if not self.use_if:
            x1 = self.cnn_features(x1)
            x2 = self.cnn_features(x2)

            x1 = x1.reshape(x1.size(0), -1)
            x2 = x2.reshape(x2.size(0), -1)
        else:
            x1_1, x1_2 = self.cnn_features(x1)
            x2_1, x2_2 = self.cnn_features(x2)

            x1_1 = x1_1.reshape(x1_1.size(0), -1)
            x1_2 = x1_2.reshape(x1_2.size(0), -1)
            x2_1 = x2_1.reshape(x2_1.size(0), -1)
            x2_2 = x2_2.reshape(x2_2.size(0), -1)

            x1 = torch.cat([x1_1, x1_2], dim=1)
            x2 = torch.cat([x2_1, x2_2], dim=1)


        x = torch.cat([x1, x2], dim=1)

        x = F.relu(self.fc(x))
        #x = self.dp(x)

        x = F.relu(self.fc2(x))
        #x = self.dp2(x)

        state = (Variable(state[0].data.to(device)), Variable(state[1].data.to(device)))

        x, n_state = self.lstm(x.unsqueeze(0), state)
        x = x.squeeze(0)

        return self.actor_policy(x), self.critic(x), n_state

    def init_state(self, device):
        """
        Returns the initial state of the model
        """
        return (torch.zeros(self.lstm_layers, 1, self.lstm_size).to(device),
                torch.zeros(self.lstm_layers, 1, self.lstm_size).to(device))

