"""
Written by Matteo Dunnhofer - 2020

Class that defines our tracker with mask generation
"""
#import sys
#sys.path.insert(0, '..')
import numpy as np
import torch
from track.Tracker import TRAS, TRAST, TRASFUST
from demotrackers.SiamMask.tools.test_my import *
from demotrackers.SiamMask.experiments.siammask_sharp.custom_my import Custom


class SiamMaskTracker(object):

    def __init__(self, gpu_id=0, args=None):
        # SiamMask setup

        self.device = torch.device('cuda', gpu_id)
        torch.backends.cudnn.benchmark = True

        self.cfg = load_config(args)
        self.siammask = Custom(anchors=self.cfg['anchors'])
        if args.resume:
            assert isfile(args.resume), 'Please download {} first.'.format(args.resume)
            self.siammask = load_pretrain(self.siammask, args.resume)

        self.siammask.eval().to(self.device)

    def init(self, image, box):
        im = np.array(image)
        x, y, w, h = box[0], box[1], box[2], box[3]
        target_pos = np.array([x + w / 2, y + h / 2])
        target_sz = np.array([w, h])
        self.state = siamese_init(im, target_pos, target_sz, self.siammask, self.cfg['hp'], device=self.device)
        #print(self.state['p'])

    def update(self, image, box):
        im = np.array(image)
        self.state['target_pos'] = np.array([box[0], box[1]])
        self.state['target_sz'] = np.array([box[2], box[3]])
        #print(self.state['window'])
        self.state = siamese_track(self.state, im, mask_enable=True, refine_enable=True, device=self.device, debug=False)  # track
        location = self.state['ploygon'].flatten()
        mask = self.state['mask'] > self.state['p'].seg_thr

        return mask, location



class TrackerMask(object):

    def __init__(self, tracker, ckpt_path, teacher_tracking=['SiamRPN'], teacher_training='POOL', lstm_update=False, gpu_id=0, args=None):
        super(TrackerMask, self).__init__()

        # tracker setup
        if tracker == 'TRAS':
            self.tracker = TRAS(ckpt_path, lstm_update=lstm_update, gpu_id=gpu_id)
        elif tracker == 'TRAST':
            self.tracker = TRAST(ckpt_path, teacher_tracking, lstm_update=lstm_update, gpu_id=gpu_id, teacher_training=teacher_training)
        elif tracker == 'TRASFUST':
            self.tracker = TRASFUST(ckpt_path, teacher_tracking, lstm_update=lstm_update, gpu_id=gpu_id, teacher_training=teacher_training)

        # SiamMask setup
        self.siammask = SiamMaskTracker(gpu_id=gpu_id, args=args)

    def init(self, image, box, **kwargs):
        self.tracker.init(image, box, **kwargs)

        # SiamMask
        self.siammask.init(image, box)

    def update(self, image):
        tracker_pred = self.tracker.update(image)

        # SiamMask
        mask, polygon = self.siammask.update(image, tracker_pred)

        return tracker_pred, mask, polygon
